# coding: utf-8

from ._check_project_name import check_project_name
from .save_popup import SavePopup
from .settings_window import SettingsWindow
from .splash_window import SplashWindow
from .structure_classes import (Settings, Nucleus, Fiber, Nuclei, Fibers,
                                GraphicalElement, SelectionBox, TableItems,
                                TableEntry, ProcessResult, ProcessError)
from .warning_window import WarningWindow
from ._check_image import check_image



# coding: utf-8

from pathlib import Path
from PIL import Image

from .util import BaseTestInterface, mock_filedialog, mock_warning_window


class Test10DeletePartial(BaseTestInterface):

    def testDeletePartial(self) -> None:
        """This test checks that the partial deletion of images using the
        checkboxes and master delete button in the files table works as
        expected."""

        # The mock selection window returns the paths to three images to load
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / f'image_{i}.jpg')
            for i in (1, 2, 3)]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Checking that the first image is displayed in the image canvas
        self.assertIsInstance(self._window._image_canvas._image, Image.Image)
        self.assertIsInstance(self._window._image_canvas._image_path, Path)
        self.assertEqual(self._window._image_canvas._image_path.name,
                         'image_1.jpg')

        # Checking that three images are referenced in the files table
        self.assertEqual(len(self._window._files_table.table_items), 3)
        self.assertEqual(
            self._window._files_table.table_items.current_entry.path.name,
            'image_1.jpg')
        self.assertCountEqual(
            [item.path.name for item in self._window._files_table.table_items],
            [f'image_{i}.jpg' for i in (1, 2, 3)])

        # Invoking the master select button to unselect all the images in the
        # files table
        self._window._select_all_button.invoke()
        # Selecting only the first image using its individual select checkbox
        (self._window._files_table.table_items.current_entry.graph_elt.
         check_button.invoke())
        # Invoking the master delete button in the files table
        self._window._delete_all_button.invoke()

        # Checking that the second image is now displayed in the image canvas
        self.assertIsInstance(self._window._image_canvas._image, Image.Image)
        self.assertIsInstance(self._window._image_canvas._image_path, Path)
        self.assertEqual(self._window._image_canvas._image_path.name,
                         'image_2.jpg')

        # Checking that only two images are left in the files table
        self.assertEqual(len(self._window._files_table.table_items), 2)
        self.assertEqual(
            self._window._files_table.table_items.current_entry.path.name,
            'image_2.jpg')
        self.assertCountEqual(
            [item.path.name for item in self._window._files_table.table_items],
            [f'image_{i}.jpg' for i in (2, 3)])

        # Invoking the master delete button in the files table
        self._window._delete_all_button.invoke()

        # Checking that no image is displayed in the image canvas
        self.assertIsNone(self._window._image_canvas._image)
        self.assertIsNone(self._window._image_canvas._image_path)

        # Checking that no image is left in the files table
        self.assertEqual(len(self._window._files_table.table_items), 0)



# coding: utf-8

import filecmp
import shutil
from pathlib import Path
from threading import Thread
from time import sleep

from .util import (BaseTestInterfaceProcessing, mock_filedialog,
                   mock_warning_window)


class Test14SaveProject(BaseTestInterfaceProcessing):

    def testSaveProject(self) -> None:
        """This test checks that the correct files are being created when
        saving a project."""

        # The mock selection window returns the path to one image to load
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / 'image_1.jpg')]
        mock_warning_window.WarningWindow.value = 1
        # The images will be saved to the existing temporary folder created
        # before starting the test
        mock_filedialog.save_folder = str(Path(self._dir.name) / 'save_folder')
        self._window._select_images()

        # Stopping the regular processing Thread
        self._window._stop_thread = True
        sleep(2)
        # Instantiating a Thread that will stop the processing loop later on
        stop_thread = Thread(target=self._stop_thread)

        # Invoking the button for starting the image processing
        self._window._process_images_button.invoke()

        # Starting the Threads for later stopping the processing loop
        self._window._stop_thread = False
        stop_thread.start()
        # Starting the processing loop in the main Thread rather than in a
        # separate one
        self._window._process_thread()
        self._window._handle_ui_queue()
        self._window._handle_ui_queue()

        # Triggering a save action using the "Save" button
        self._window._save_button.invoke()

        # Checking that all the files that should be saved were indeed created
        save_path = Path(mock_filedialog.save_folder)
        self.assertTrue(save_path.exists())
        self.assertTrue(save_path.is_dir())
        self.assertTrue((save_path / 'settings.pickle').exists())
        self.assertTrue((save_path / 'save_folder.xlsx').exists())
        self.assertTrue((save_path / 'data.pickle').exists())
        self.assertTrue((save_path / 'Original Images').exists())
        self.assertTrue((save_path / 'Original Images').is_dir())
        self.assertGreater(len(tuple((save_path /
                                      'Original Images').iterdir())), 0)

        # Copying all the files that were just saved to a different folder
        shutil.copytree(save_path, save_path / 'copy')

        # Triggering for a second time a save action using the "Save" button
        self._window._save_button.invoke()

        # Checking that the recorded files are still the same as the first ones
        comp = filecmp.cmpfiles(save_path, save_path / 'copy',
                                ('data.pickle', 'settings.pickle',
                                 'save_folder.xlsx',
                                 'Original Images/image_1.jpg'), shallow=False)
        self.assertCountEqual(('data.pickle', 'settings.pickle',
                               'save_folder.xlsx',
                               'Original Images/image_1.jpg'), comp[0])
        self.assertFalse(comp[1] or comp[2])

        # Adding a single nucleus by clicking on the image canvas, so that the
        # project is now different
        self._window._image_canvas._canvas.event_generate(
            '<ButtonPress-1>', when="now", x=10, y=10)
        self._window._image_canvas._canvas.event_generate(
            '<ButtonRelease-1>', when="now", x=10, y=10)
        self._window._show_nuclei_check_button.invoke()

        # Triggering a third save action using the "Save" button
        self._window._save_button.invoke()

        # Checking that some recorded files are now different to the original
        # due to the modification
        comp = filecmp.cmpfiles(save_path, save_path / 'copy',
                                ('data.pickle', 'settings.pickle',
                                 'save_folder.xlsx',
                                 'Original Images/image_1.jpg'), shallow=False)
        self.assertCountEqual(('Original Images/image_1.jpg',), comp[0])
        self.assertCountEqual(('data.pickle', 'save_folder.xlsx',
                               'settings.pickle'), comp[1])
        self.assertFalse(comp[2])



# coding: utf-8

from .util import BaseTestInterface, mock_filedialog


class Test02LoadNoImages(BaseTestInterface):

    def testLoadNoImages(self) -> None:
        """This test checks that MyoFInDer successfully handles the situation
        when the user aborts an image selection for loading into the interface.
        """

        # None so that no image is returned by the mock selection window
        mock_filedialog.file_name = None
        self._window._select_images()

        # Checking that no image was loaded in the image canvas and file table
        self.assertIsNone(self._window._image_canvas._image)
        self.assertIsNone(self._window._image_canvas._image_path)
        self.assertEqual(len(self._window._files_table.table_items), 0)



# coding: utf-8

from tkinter import messagebox
from pathlib import Path

forbidden_chars = ['/', '\\', '>', '<', ':', '"', '|', '?', '*', '.']
forbidden_names = ['CON', 'PRN', 'AUX', 'NUL', 'COM1', 'COM2', 'COM3', 'COM4',
                   'COM5', 'COM6', 'COM7', 'COM8', 'COM9', 'LPT1', 'LPT2',
                   'LPT3', 'LPT4', 'LPT5', 'LPT6', 'LPT7', 'LPT8', 'LPT9']


def check_project_name(path: Path) -> bool:
    """Takes a path where the user wants to save the project as an input, and
    determines whether it is valid. If so, returns True, and False otherwise.

    Args:
        path: The path chosen by the user for saving the project.

    Returns:
        True if the project path is valid and the project can be saved there,
        False otherwise.
    """

    # First, check that there's no forbidden character
    if any(char in forbidden_chars for char in path.name):
        forbidden = ', '.join(set(char for char in path.name
                                  if char in forbidden_chars))
        messagebox.showwarning("Invalid project name",
                               f"The name contains the following forbidden "
                               f"characters :\n{forbidden}")
        return False

    # Then, check that the name is not reserved
    if any(path.name == forbidden for forbidden in forbidden_names):
        messagebox.showwarning("Invalid project name",
                               "This name might be reserved by the OS !\n"
                               "Please choose another one.")
        return False

    # Then, if the name is fine, check that it is not already used
    if path.is_dir() and path.exists() and any(file.suffix == '.pickle'
                                               for file in path.iterdir()):
        messagebox.showwarning("Invalid project name",
                               "This project already exists !\n"
                               "Please choose another one.")
        return False

    return True



# coding: utf-8

from pathlib import Path
from time import sleep
from threading import Thread

from .util import (BaseTestInterfaceProcessing, mock_warning_window,
                   mock_filedialog)


class Test21ProcessVaryChannels(BaseTestInterfaceProcessing):

    def testProcessVaryChannels(self) -> None:
        """This test checks that the image processing gives a similar output
        with all the possible combinations of channels colors.

        To this end, a same image was duplicated but with its colors shuffled
        across all the possible channel combinations.
        """

        # The mock selection window returns the path to one image to load
        # The nuclei are on the blue channel, the fibers on the green channel
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / 'image_1.jpg')]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Stopping the regular processing Thread
        self._window._stop_thread = True
        sleep(2)
        # Instantiating a Thread that will stop the processing loop later on
        stop_thread = Thread(target=self._stop_thread)

        # Invoking the creation of a settings window
        index = self._window._settings_menu.index("Settings")
        self._window._settings_menu.invoke(index)
        # Setting the nuclei channel to blue, the fibers channel to green
        self._window._settings_window._nuclei_colour_r1.invoke()
        self._window._settings_window._fiber_colour_r2.invoke()
        # Closing the settings window
        self._window._settings_window.destroy()

        # Invoking the button for starting the image processing
        self._window._process_images_button.invoke()

        # Starting the Threads for later stopping the processing loop
        self._window._stop_thread = False
        stop_thread.start()
        # Re-starting the processing loop in the main Thread
        self._window._process_thread()
        self._window._handle_ui_queue()
        self._window._handle_ui_queue()

        # Reading the processing output to later compare it with other runs
        nuc = len(self._window._files_table.table_items.entries[0].nuclei)
        nuc_in = (self._window._files_table.table_items.entries[0].
                  nuclei.nuclei_in_count)
        nuc_out = (self._window._files_table.table_items.entries[0].
                   nuclei.nuclei_out_count)
        fib = len(self._window._files_table.table_items.entries[0].fibers)
        area = self._window._files_table.table_items.entries[0].fibers.area

        # Making sure that some nuclei and some fibers were detected
        self.assertGreater(nuc, 0)
        self.assertGreater(nuc_in, 0)
        self.assertGreater(nuc_out, 0)
        self.assertGreater(fib, 0)
        self.assertGreater(area, 0)

        # Removing the existing image from the image canvas and the files table
        self._window._delete_all_button.invoke()

        # The nuclei are on the green channel, the fibers on the blue channel
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / 'image_4.jpg')]
        self._window._select_images()

        # Invoking the creation of a settings window
        index = self._window._settings_menu.index("Settings")
        self._window._settings_menu.invoke(index)
        # Setting the nuclei channel to green, the fibers channel to blue
        self._window._settings_window._nuclei_colour_r2.invoke()
        self._window._settings_window._fiber_colour_r1.invoke()
        # Closing the settings window
        self._window._settings_window.destroy()

        # Re-instantiating a Thread that will stop the processing loop later on
        stop_thread = Thread(target=self._stop_thread)
        # Invoking the button for starting the image processing
        self._window._process_images_button.invoke()
        # Starting the Threads for later stopping the processing loop
        self._window._stop_thread = False
        stop_thread.start()
        # Re-starting the processing loop in the main Thread
        self._window._process_thread()
        self._window._handle_ui_queue()
        self._window._handle_ui_queue()

        # Checking that the output of the processing is reasonably close to the
        # one with the regular colors
        self.assertLessEqual(
            abs(nuc - len(self._window._files_table.table_items.
                          entries[0].nuclei)), 5)
        self.assertLessEqual(
            abs(nuc_in - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_in_count), 8)
        self.assertLessEqual(
            abs(nuc_out - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_out_count), 6)
        self.assertLessEqual(
            abs(fib - len(self._window._files_table.table_items.
                          entries[0].fibers)), 3)
        self.assertLessEqual(
            abs(area - self._window._files_table.table_items.entries[0].
                fibers.area), 0.05)

        # Removing the existing image from the image canvas and the files table
        self._window._delete_all_button.invoke()

        # The nuclei are on the blue channel, the fibers on the red channel
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / 'image_5.jpg')]
        self._window._select_images()

        # Invoking the creation of a settings window
        index = self._window._settings_menu.index("Settings")
        self._window._settings_menu.invoke(index)
        # Setting the nuclei channel to blue, the fibers channel to red
        self._window._settings_window._nuclei_colour_r1.invoke()
        self._window._settings_window._fiber_colour_r3.invoke()
        # Closing the settings window
        self._window._settings_window.destroy()

        # Re-instantiating a Thread that will stop the processing loop later on
        stop_thread = Thread(target=self._stop_thread)
        # Invoking the button for starting the image processing
        self._window._process_images_button.invoke()
        # Starting the Threads for later stopping the processing loop
        self._window._stop_thread = False
        stop_thread.start()
        # Re-starting the processing loop in the main Thread
        self._window._process_thread()
        self._window._handle_ui_queue()
        self._window._handle_ui_queue()

        # Checking that the output of the processing is reasonably close to the
        # one with the regular colors
        self.assertLessEqual(
            abs(nuc - len(self._window._files_table.table_items.
                          entries[0].nuclei)), 5)
        self.assertLessEqual(
            abs(nuc_in - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_in_count), 5)
        self.assertLessEqual(
            abs(nuc_out - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_out_count), 5)
        self.assertLessEqual(
            abs(fib - len(self._window._files_table.table_items.
                          entries[0].fibers)), 3)
        self.assertLessEqual(
            abs(area - self._window._files_table.table_items.entries[0].
                fibers.area), 0.05)

        # Removing the existing image from the image canvas and the files table
        self._window._delete_all_button.invoke()

        # The nuclei are on the green channel, the fibers on the red channel
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / 'image_6.jpg')]
        self._window._select_images()

        # Invoking the creation of a settings window
        index = self._window._settings_menu.index("Settings")
        self._window._settings_menu.invoke(index)
        # Setting the nuclei channel to green, the fibers channel to red
        self._window._settings_window._nuclei_colour_r2.invoke()
        self._window._settings_window._fiber_colour_r3.invoke()
        # Closing the settings window
        self._window._settings_window.destroy()

        # Re-instantiating a Thread that will stop the processing loop later on
        stop_thread = Thread(target=self._stop_thread)
        # Invoking the button for starting the image processing
        self._window._process_images_button.invoke()
        # Starting the Threads for later stopping the processing loop
        self._window._stop_thread = False
        stop_thread.start()
        # Re-starting the processing loop in the main Thread
        self._window._process_thread()
        self._window._handle_ui_queue()
        self._window._handle_ui_queue()

        # Checking that the output of the processing is reasonably close to the
        # one with the regular colors
        self.assertLessEqual(
            abs(nuc - len(self._window._files_table.table_items.
                          entries[0].nuclei)), 5)
        self.assertLessEqual(
            abs(nuc_in - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_in_count), 5)
        self.assertLessEqual(
            abs(nuc_out - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_out_count), 8)
        self.assertLessEqual(
            abs(fib - len(self._window._files_table.table_items.
                          entries[0].fibers)), 3)
        self.assertLessEqual(
            abs(area - self._window._files_table.table_items.entries[0].
                fibers.area), 0.05)

        # Removing the existing image from the image canvas and the files table
        self._window._delete_all_button.invoke()

        # The nuclei are on the red channel, the fibers on the blue channel
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / 'image_7.jpg')]
        self._window._select_images()

        # Invoking the creation of a settings window
        index = self._window._settings_menu.index("Settings")
        self._window._settings_menu.invoke(index)
        # Setting the nuclei channel to red, the fibers channel to blue
        self._window._settings_window._nuclei_colour_r3.invoke()
        self._window._settings_window._fiber_colour_r1.invoke()
        # Closing the settings window
        self._window._settings_window.destroy()

        # Re-instantiating a Thread that will stop the processing loop later on
        stop_thread = Thread(target=self._stop_thread)
        # Invoking the button for starting the image processing
        self._window._process_images_button.invoke()
        # Starting the Threads for later stopping the processing loop
        self._window._stop_thread = False
        stop_thread.start()
        # Re-starting the processing loop in the main Thread
        self._window._process_thread()
        self._window._handle_ui_queue()
        self._window._handle_ui_queue()

        # Checking that the output of the processing is reasonably close to the
        # one with the regular colors
        self.assertLessEqual(
            abs(nuc - len(self._window._files_table.table_items.
                          entries[0].nuclei)), 5)
        self.assertLessEqual(
            abs(nuc_in - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_in_count), 5)
        self.assertLessEqual(
            abs(nuc_out - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_out_count), 5)
        self.assertLessEqual(
            abs(fib - len(self._window._files_table.table_items.
                          entries[0].fibers)), 3)
        self.assertLessEqual(
            abs(area - self._window._files_table.table_items.entries[0].
                fibers.area), 0.05)

        # Removing the existing image from the image canvas and the files table
        self._window._delete_all_button.invoke()

        # The nuclei are on the red channel, the fibers on the green channel
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / 'image_8.jpg')]
        self._window._select_images()

        # Invoking the creation of a settings window
        index = self._window._settings_menu.index("Settings")
        self._window._settings_menu.invoke(index)
        # Setting the nuclei channel to red, the fibers channel to green
        self._window._settings_window._nuclei_colour_r3.invoke()
        self._window._settings_window._fiber_colour_r2.invoke()
        # Closing the settings window
        self._window._settings_window.destroy()

        # Re-instantiating a Thread that will stop the processing loop later on
        stop_thread = Thread(target=self._stop_thread)
        # Invoking the button for starting the image processing
        self._window._process_images_button.invoke()
        # Starting the Threads for later stopping the processing loop
        self._window._stop_thread = False
        stop_thread.start()
        # Re-starting the processing loop in the main Thread
        self._window._process_thread()
        self._window._handle_ui_queue()
        self._window._handle_ui_queue()

        # Checking that the output of the processing is reasonably close to the
        # one with the regular colors
        self.assertLessEqual(
            abs(nuc - len(self._window._files_table.table_items.
                          entries[0].nuclei)), 6)
        self.assertLessEqual(
            abs(nuc_in - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_in_count), 6)
        self.assertLessEqual(
            abs(nuc_out - self._window._files_table.table_items.entries[0].
                nuclei.nuclei_out_count), 6)
        self.assertLessEqual(
            abs(fib - len(self._window._files_table.table_items.
                          entries[0].fibers)), 3)
        self.assertLessEqual(
            abs(area - self._window._files_table.table_items.entries[0].
                fibers.area), 0.05)



# coding: utf-8

from pathlib import Path
from threading import Thread
from time import sleep

from .util import (BaseTestInterfaceProcessing, mock_filedialog,
                   mock_warning_window)


class Test13IndicatorsDisplay(BaseTestInterfaceProcessing):

    def testIndicatorsDisplay(self) -> None:
        """This test checks that the checkboxes driving the display of the
        detected nuclei and fibers are working as expected."""

        # The mock selection window returns the path to one image to load
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / 'image_1.jpg')]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Stopping the regular processing Thread
        self._window._stop_thread = True
        sleep(2)
        # Instantiating a Thread that will stop the processing loop later on
        stop_thread = Thread(target=self._stop_thread)

        # Checking that no Tkinter objects representing nuclei are present on
        # the image canvas
        self.assertTrue(not any(nuc.tk_obj is not None for nuc
                                in self._window._image_canvas._nuclei))
        # The fiber overlay should be present whatever happens though
        self.assertIsNotNone(self._window._image_canvas._fib_overlay_tk)

        # Invoking the button for starting the image processing
        self._window._process_images_button.invoke()

        # Starting the Threads for later stopping the processing loop
        self._window._stop_thread = False
        stop_thread.start()
        # Starting the processing loop in the main Thread rather than in a
        # separate one
        self._window._process_thread()
        self._window._handle_ui_queue()
        self._window._handle_ui_queue()

        display_nuc = self._window.settings.show_nuclei.get()
        display_fib = self._window.settings.show_fibers.get()

        self.assertTrue(all(nuc.tk_obj is not None for nuc
                            in self._window._image_canvas._nuclei))
        if display_nuc:
            self.assertTrue(all(self._window._image_canvas._canvas.itemcget(
                nuc.tk_obj, "state") == 'normal' for nuc
                in self._window._image_canvas._nuclei))
        else:
            self.assertTrue(all(self._window._image_canvas._canvas.itemcget(
                nuc.tk_obj, "state") == 'hidden' for nuc
                in self._window._image_canvas._nuclei))

        self.assertIsNotNone(self._window._image_canvas._fib_overlay_tk)
        if display_fib:
            self.assertEqual(self._window._image_canvas._canvas.itemcget(
                self._window._image_canvas._fib_overlay_idx, "state"),
                'normal')
        else:
            self.assertEqual(self._window._image_canvas._canvas.itemcget(
                self._window._image_canvas._fib_overlay_idx, "state"),
                'hidden')

        self._window._show_nuclei_check_button.invoke()
        self._window._show_fibers_check_button.invoke()

        if display_nuc:
            self.assertTrue(all(self._window._image_canvas._canvas.itemcget(
                nuc.tk_obj, "state") == 'hidden' for nuc
                in self._window._image_canvas._nuclei))
        else:
            self.assertTrue(all(self._window._image_canvas._canvas.itemcget(
                nuc.tk_obj, "state") == 'normal' for nuc
                in self._window._image_canvas._nuclei))

        self.assertIsNotNone(self._window._image_canvas._fib_overlay_tk)
        if display_fib:
            self.assertEqual(self._window._image_canvas._canvas.itemcget(
                self._window._image_canvas._fib_overlay_idx, "state"),
                'hidden')
        else:
            self.assertEqual(self._window._image_canvas._canvas.itemcget(
                self._window._image_canvas._fib_overlay_idx, "state"),
                'normal')



# coding: utf-8

from copy import deepcopy

from .util import BaseTestInterface


class Test18ChangeSettings(BaseTestInterface):

    def testChangeSettings(self) -> None:
        """This test checks that interacting with the settings-related
        graphical objects does change the internal variables storing the
        settings values."""

        # Copying the initial state of the settings for later comparing it
        init_settings = deepcopy(self._window.settings.get_all())

        # Checking that no settings window was created so far
        self.assertIsNone(self._window._settings_window)

        # Invoking the creation of a settings window
        index = self._window._settings_menu.index("Settings")
        self._window._settings_menu.invoke(index)

        # Checking that the settings window was now created
        self.assertIsNotNone(self._window._settings_window)

        # Modifying the nuclei colour setting value
        if init_settings['nuclei_colour'] == 'blue':
            self._window._settings_window._nuclei_colour_r2.invoke()
        elif init_settings['nuclei_colour'] == 'green':
            self._window._settings_window._nuclei_colour_r3.invoke()
        elif init_settings['nuclei_colour'] == 'red':
            self._window._settings_window._nuclei_colour_r1.invoke()

        # Modifying the fiber colour setting value
        if init_settings['fiber_colour'] == 'blue':
            self._window._settings_window._fiber_colour_r2.invoke()
        elif init_settings['fiber_colour'] == 'green':
            self._window._settings_window._fiber_colour_r3.invoke()
        elif init_settings['fiber_colour'] == 'red':
            self._window._settings_window._fiber_colour_r1.invoke()

        # Modifying the save overlay setting value
        if init_settings['save_overlay']:
            self._window._settings_window._overlay_off_button.invoke()
        else:
            self._window._settings_window._overlay_on_button.invoke()

        # Modifying the minimum fiber intensity setting value
        min_fib_int = init_settings['minimum_fiber_intensity']
        if int(min_fib_int - 1) > 0:
            self._window._settings_window._min_fib_int_slider.set(
                int(min_fib_int - 1))
        else:
            self._window._settings_window._min_fib_int_slider.set(
                int(min_fib_int + 1))

        # Modifying the maximum fiber intensity setting value
        max_fib_int = init_settings['maximum_fiber_intensity']
        if int(max_fib_int + 1) < 255:
            self._window._settings_window._max_fib_int_slider.set(
                int(max_fib_int + 1))
        else:
            self._window._settings_window._max_fib_int_slider.set(
                int(max_fib_int - 1))

        # Modifying the minimum nucleus intensity setting value
        min_nuc_int = init_settings['minimum_nucleus_intensity']
        if int(min_nuc_int - 1) > 0:
            self._window._settings_window._min_nuc_int_slider.set(
                int(min_nuc_int - 1))
        else:
            self._window._settings_window._min_nuc_int_slider.set(
                int(min_nuc_int + 1))

        # Modifying the maximum nucleus intensity setting value
        max_nuc_int = init_settings['maximum_nucleus_intensity']
        if int(max_nuc_int + 1) < 255:
            self._window._settings_window._max_nuc_int_slider.set(
                int(max_nuc_int + 1))
        else:
            self._window._settings_window._max_nuc_int_slider.set(
                int(max_nuc_int - 1))

        # Modifying the minimum nucleus diameter setting value
        min_nuc_diam = init_settings['minimum_nuc_diameter']
        if int(min_nuc_diam - 1) > 0:
            self._window._settings_window._min_nuc_diam_slider.set(
                int(min_nuc_diam - 1))
        else:
            self._window._settings_window._min_nuc_diam_slider.set(
                int(min_nuc_diam + 1))

        # Modifying the minimum nuclei count setting value
        min_nuc_count = init_settings['minimum_nuclei_count']
        if int(min_nuc_count - 1) > 0:
            self._window._settings_window._count_slider.set(
                int(min_nuc_count - 1))
        else:
            self._window._settings_window._count_slider.set(
                int(min_nuc_count + 1))

        # Modifying the channels display settings values
        self._window._red_channel_check_button.invoke()
        self._window._green_channel_check_button.invoke()
        self._window._blue_channel_check_button.invoke()

        # Modifying the results display settings values
        self._window._show_nuclei_check_button.invoke()
        self._window._show_fibers_check_button.invoke()

        # Closing the settings window and checking that it was destroyed
        self._window._settings_window.destroy()
        self.assertIsNone(self._window._settings_window)

        # Checking that the same settings are still available
        self.assertTrue(all(key in init_settings for key
                            in self._window.settings.get_all()))

        # Checking that no setting value was left unchanged
        self.assertFalse(any(val_1 == val_2 for (_, val_1), (_, val_2)
                             in zip(init_settings.items(),
                                    self._window.settings.get_all().items())))
